'''Unittest for reposearch'''

import sys
import os
import os.path as op
import shutil
from exceptions import *
import errno

import reposearch as rs
import unittest

class Setup(object):
    dir = "reposearch_data"
    def setup(self):
        spec = """bzr       compound/bzr
                  git       compound/git
                  gitbare   compound/git.git
                  bzr       compound/dir/bzr
                  git       compound/dir/git
                  gitbare   compound/dir/git.git
                  git       git/a
                  git       git/b
                  file      git/no/empty
                  git       git/dir/a
                  git       git/dir/b
                  file      git/dir/no/empty
                  git       gitbare/a
                  gitbare   gitbare/a.git
                  gitbare   gitbare/b.git
                  gitbare   gitbare/dir/a.git
                  gitbare   gitbare/dir/b.git
                  file      gitbare/no.git/empty
                  git       repoinner/git
                  gitbare   repoinner/git.git"""
        if not op.isdir(self.dir):
            raise RuntimeError("dir %s not found" %(self.dir))
        basepath = self.dir
        def touch(file):
            """creates all dirs up to basepath+dirname(file)
            including the empty file itself"""
            our_file = op.join(basepath, file)
            path = op.dirname(our_file)
            if path:
                try: os.makedirs(path)
                except OSError,e:
                    if e.errno != errno.EEXIST:
                        raise e
            tmp = open(our_file, "a")
            tmp.close()
            tmp = None
        actions = {
                "bzr": lambda d: touch(op.join(d, ".bzr", "empty")),
                "git": lambda d: touch(op.join(d, ".git", "config")),
                "gitbare": lambda d: touch(op.join(d, "config")),
                "file": lambda d: touch(d)
            }
        for i,j in [ i.split() for i in spec.splitlines() ]:
            actions[i](j)
    def cleanup(self):
        map(lambda d: shutil.rmtree(op.join(self.dir, d)), os.listdir(self.dir))


class Base(unittest.TestCase):
    dir = "reposearch_data"
    data = []
    def inner(self, f, append_dir=None):
        '''Execute f() inside self.dir'''
        old = op.abspath(os.curdir)
        dir = self.dir
        if append_dir != None:
            dir = op.join(dir, append_dir)
        os.chdir(dir)
        try:
            tmp = f()
        finally:
            os.chdir(old)
        return tmp
    def innerT(self, walkfun, testdir, repotype, filter_fun=None, append_dir=None):
        '''Test walkfun()/2 inside testdir on repotype'''
        filter_fun = filter_fun or rs.repo_types[repotype]
        should = self.data
        curr = sorted(self.inner(lambda: list(walkfun(testdir, filter_fun)), append_dir))
        self.assertEqual(should, curr)
    def outerT(self, walkfun, testdir, repotype, filter_fun=None):
        '''Test walkfun()/2 from outside testdir on repotype'''
        filter_fun = filter_fun or rs.repo_types[repotype]
        should = sorted([ op.join(self.dir, i) for i in self.data ])
        curr = sorted(list(walkfun(op.join(self.dir, testdir), filter_fun)))
        self.assertEqual(should, curr)

class GitFlat(Base):
    data = sorted('git/a git/b'.split())
    def testFlatOuter(self):
        self.outerT(rs.walkrepos_flat, 'git', 'git')
    def testFlatInner(self):
        self.innerT(rs.walkrepos_flat, 'git', 'git')

class GitRec(Base):
    data = sorted('git/a git/b git/dir/a git/dir/b'.split())
    def testRecOuter(self):
        self.outerT(rs.walkrepos, 'git', 'git')
    def testRecInner(self):
        self.innerT(rs.walkrepos, 'git', 'git')

class GitBareFlat(Base):
    data = sorted('gitbare/a.git gitbare/b.git'.split())
    def testFlatOuter(self):
        self.outerT(rs.walkrepos_flat, 'gitbare', 'gitbare')
    def testFlatInner(self):
        self.innerT(rs.walkrepos_flat, 'gitbare', 'gitbare')

class GitBareRec(Base):
    data = sorted('gitbare/a.git gitbare/b.git gitbare/dir/a.git gitbare/dir/b.git'.split())
    def testRecOuter(self):
        self.outerT(rs.walkrepos, 'gitbare', 'gitbare')
    def testRecInner(self):
        self.innerT(rs.walkrepos, 'gitbare', 'gitbare')

class Compound(unittest.TestCase):
    def test_parse_filter_spec(self):
        data = 'git,gitbare,bzr'
        should = data.split(',')
        curr = rs.parse_filter_spec(data)
        self.assertEqual(should, curr)

class CompoundFlat(Base):
    data = sorted([ op.join('compound', i) for i in 'git git.git bzr'.split()])
    def test_outer(self):
        filter_fun = rs.create_filter('git gitbare bzr'.split())
        self.outerT(rs.walkrepos_flat, 'compound', None, filter_fun)
    def test_inner(self):
        filter_fun = rs.create_filter('git gitbare bzr'.split())
        self.innerT(rs.walkrepos_flat, 'compound', None, filter_fun)

class CompoundRec(Base):
    data = sorted([ op.join('compound', i) for i in 'git git.git bzr dir/git dir/git.git dir/bzr'.split()])
    def test_outer(self):
        filter_fun = rs.create_filter('git gitbare bzr'.split())
        self.outerT(rs.walkrepos, 'compound', None, filter_fun)
    def test_inner(self):
        filter_fun = rs.create_filter('git gitbare bzr'.split())
        self.innerT(rs.walkrepos, 'compound', None, filter_fun)

class RepoInner(Base):
    data = ['.']
    def test_git_within_flat(self):
        self.innerT(rs.walkrepos_flat, '.', 'git', append_dir='repoinner/git')
    def test_git_within_rec(self):
        self.innerT(rs.walkrepos, '.', 'git', append_dir='repoinner/git')
    def test_gitbare_within_flat(self):
        self.innerT(rs.walkrepos_flat, '.', 'gitbare', append_dir='repoinner/git.git')
    def test_gitbare_within_rec(self):
        self.innerT(rs.walkrepos, '.', 'gitbare', append_dir='repoinner/git.git')


if __name__ == "__main__":
    a=Setup()

    if "--cleanup" in sys.argv:
        a.cleanup()
        sys.argv.remove("--cleanup")
        sys.exit(0)
    a.setup()
    unittest.main()
